
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import sys
import os
import __builtin__

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from threading import Semaphore
from collections import deque


from utils.ClassFactory import class_from_name

from KnowledgeBase import KnowledgeBase
from audit_subsystem.AuditManager import AuditManager
from config_subsystem.ConfigManager import ConfigManager
from discovery_subsystem.DiscoveryManager import DiscoveryManager
from output_subsystem.OutputManager import OutputManager
from request_subsystem.RequestManager import RequestManager
from constants.option_types import LIST_STR

class RabbitCore(object):
    """Main class of the whole framework.
    
    Its purpose is to control the flow of the whole auditing process.
    
    """
    
    _core = None
    
    def __init__(self):
        
        # First create the ConfigManager and create own options
        self._config_manager = ConfigManager(self)
        self._options = (
            self._config_manager.create_option('core', 'audit_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'request_generator_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'information_finder_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'finger_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'discovery_filter_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'request_filter_plugins', [], LIST_STR),
            self._config_manager.create_option('core', 'output_plugins', [], LIST_STR),
        )
        
        # Instantiate the rest of the managers
        self._output_manager = OutputManager(self)
        self._request_manager = RequestManager(self)
        self._discovery_manager = DiscoveryManager(self)
        self._audit_manager = AuditManager(self)
        
        # Initialize the RabbitCore's attributes
        self._state = 0 # Set stopped state.
        self._to_audit = None
        self._paused = Semaphore()
    
    def start(self):
        """Start and control the flow of the auditing process."""
        self._state = 1 # Set running state.
        self._initialize_environment()
        self.get_output_manager().log("Starting Discovery process")
        self._discovery_manager.start()
        self.get_output_manager().log("Finished Discovery process")
        self.get_output_manager().log("Starting Auditing process")
        self._audit_manager.start()
        self.get_output_manager().log("Finished Auditing process")
        self._clean()
        self._state = 0
    
    def pause(self):
        """Pause and unpause the auditing process"""
        if self._state == 1:
            self._paused.acquire()
            self._state = 2 # Set paused state.
        else:
            self._paused.release()
            self._state = 1 # Set running state.
    
    def stop(self):
        """Stop the auditing process"""
        tmp = self._state
        self._state = 0
        if tmp == 2:
            self._paused.release()
    
    def get_config_manager(self):
        return self._config_manager
    
    def get_output_manager(self):
        return self._output_manager
    
    def get_audit_manager(self):
        return self._audit_manager
    
    def get_discovery_manager(self):
        return self._discovery_manager
    
    def get_request_manager(self):
        return self._request_manager
    
    def get_to_audit(self):
        return self._to_audit
    
    def get_knowledge_base(self):
        return self._knowledge_base
    
    def get_options(self):
        return self._options
    
    def get_option(self, name):
        return [x for x in self._options if x.get_variable() == name][0]
    
    def get_state(self):
        return self._state
    
    def _initialize_environment(self):
        """Instantiate plugins, solve dependencies.
        
        Leave all the things in order prior doing the audit.
        
        """
        
        self._to_audit = deque()
        self._knowledge_base = KnowledgeBase()
        
        # Instantiate output plugins, which don't have dependencies
        output_plugins = [class_from_name('plugins.output.'+cl)() for cl in self.get_option('output_plugins').get_value()]
        self._output_manager.initialize(output_plugins)
        
        # Other plugins may have dependencies, we need to build a
        # dependency graph and do a topologic sort.
        class_category_dict = {
            'audit_plugins': [class_from_name('plugins.audit.'+cl) for cl in self.get_option('audit_plugins').get_value()],
            'discovery_consumer_plugins': [class_from_name('plugins.discovery.request_generator.'+cl) for cl in self.get_option('request_generator_plugins').get_value()] \
                                        + [class_from_name('plugins.discovery.information_finder.'+cl) for cl in self.get_option('information_finder_plugins').get_value()],
            'discovery_finger_plugins': [class_from_name('plugins.discovery.finger.'+cl) for cl in self.get_option('finger_plugins').get_value()],
            'discovery_filter_plugins': [class_from_name('plugins.discovery.filter.'+cl) for cl in self.get_option('discovery_filter_plugins').get_value()],
            'request_filter_plugins': [class_from_name('plugins.request.filter.'+cl) for cl in self.get_option('request_filter_plugins').get_value()],
        }
        
        plugin_instances = {
            'audit_plugins': [],
            'discovery_consumer_plugins': [],
            'discovery_finger_plugins': [],
            'discovery_filter_plugins': [],
            'request_filter_plugins': [],
        }
        
        plugins_class_to_process = []
        plugins_class_to_process.extend(class_category_dict['audit_plugins'])
        plugins_class_to_process.extend(class_category_dict['discovery_consumer_plugins'])
        plugins_class_to_process.extend(class_category_dict['discovery_finger_plugins'])
        plugins_class_to_process.extend(class_category_dict['discovery_filter_plugins'])
        plugins_class_to_process.extend(class_category_dict['request_filter_plugins'])
        
        dependency_graph = {}
        while plugins_class_to_process:
            class_in_process = plugins_class_to_process.pop(0)
            for dependency in class_in_process.get_class_dependencies():
                dep_class = class_from_name(dependency)
                if dep_class not in class_category_dict[dep_class.get_class_type() + '_plugins']:
                    class_category_dict[dep_class.get_class_type() + '_plugins'].append(dep_class)
                    plugins_class_to_process.append(dep_class)
                updated_dependency = dependency_graph.get(dep_class, [None, []])
                updated_dependency[1].append(class_in_process)
                dependency_graph[dep_class] = updated_dependency
            dep_record = dependency_graph.get(class_in_process, [None,[]])
            dep_record[0] = len(class_in_process.get_class_dependencies())
            dependency_graph[class_in_process] = dep_record
        
        while dependency_graph:
            class_to_process = [x for x in dependency_graph if dependency_graph[x][0] == 0][0]
            dependant_list = dependency_graph.pop(class_to_process)[1]
            
            plugin_instances[class_to_process.get_class_type() + '_plugins'].append(class_to_process())
            for dependant in dependant_list:
                dependency_graph[dependant][0] -= 1
        
        self._request_manager.set_filter_plugins(plugin_instances['request_filter_plugins'])
        self._discovery_manager.set_finger_plugins(plugin_instances['discovery_finger_plugins'])
        self._discovery_manager.set_consumer_plugins(plugin_instances['discovery_consumer_plugins'])
        self._discovery_manager.set_filter_plugins(plugin_instances['discovery_filter_plugins'])
        self._audit_manager.set_audit_plugins(plugin_instances['audit_plugins'])
    
    def _clean(self):
        """ Clean the attributes after an audition was finished or stopped"""
        self._output_manager.clean()
        self._request_manager.clean()
        self._discovery_manager.clean()
        self._audit_manager.clean()
        self._paused = Semaphore()

__builtin__.rabbit_core = RabbitCore()
rabbit_core.get_config_manager().load()