
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import xml.dom.minidom
import copy
import os
import pickle

from Option import Option
from utils.ClassFactory import class_from_name
from core.constants.option_types import STRING

class ConfigManager(object):
    
    def __init__(self, core):
        
        self._file_name = os.path.dirname(os.path.realpath(__file__)) + "/config.xml"
        
        self._configuration = {}
        self._load_static_configuration()
        self._core = core
        self._private_categories = ['config_manager', 'config_plugin']
        self._options = (
            self.create_option('config_manager', 'session', 'default', STRING),
            self.create_option('config_manager', 'config_plugin', 'XMLConfigPlugin', STRING),
        )
        self._config_plugin = None
    
    def save(self, session_name=''):
        
        session_option = self.get_option('session')
        if session_name != '':
            session_option.set_value(session_name)
        
        self._save_static_configuration()
        
        tmp_vars = copy.copy(self._configuration)
        for category in [x for x in self._private_categories if x in self._configuration]:
            del tmp_vars[category]
        self.get_config_plugin().save(session_option.get_value(), tmp_vars)
    
    def load(self, session_name=''):
        
        session_option = self.get_option('session')
        if session_name != '':
            session_option.set_value(session_name)
        
        self.get_config_plugin().load(session_option.get_value())
    
    def create_option(self, category, variable, value, value_class):
        option = self._get(category, variable)
        if option is None:
            option = Option(self, category, variable, value, value_class)
            self._set(option)
        return option
    
    def get_options(self):
        return self._options
    
    def get_option(self, name):
        return [x for x in self._options if x.get_variable() == name][0]
    
    def get_config_plugin(self):
        if self._config_plugin is None:
            self._config_plugin = class_from_name("plugins.config." + self.get_option('config_plugin').get_value())()
        
        return self._config_plugin
    
    def _get(self, category, variable):
        try:
            return self._configuration[category][variable]
        except:
            return None
    
    def _set(self, option):
        
        category = option.get_category()
        
        #si no existe la categoria la creamos con su diccionario de variables vacio
        if not self._exist_category(category):
            self._configuration[category] = {}
        
        self._configuration[category][option.get_variable()] = option
    
    def _exist_category(self, category):
        return category in self._configuration
  
    def _load_static_configuration(self):
        dom_tree = xml.dom.minidom.parse(self._file_name).firstChild
        
        categories = dom_tree.childNodes
        for category in categories:
            if category.nodeName == "#text":
                continue
            
            definitions = {}
            
            variables = category.childNodes
            
            for variable in variables:
                if variable.nodeName == "#text":
                    continue
                
                type  = variable.getAttribute("class")
                value = pickle.loads(variable.getAttribute("value").decode('base64'))
                option = self.create_option(category.nodeName, variable.nodeName, value, type)
                option.set_value(value)
    
    def _save_static_configuration(self):
        dom_tree = xml.dom.minidom.Document()
        
        root_node = dom_tree.createElement("config")
        
        
        for category in self._private_categories:
            category_node = dom_tree.createElement(category)
            
            variables = self._configuration.get(category, {})
            for variable in variables:
                variable_node = dom_tree.createElement(variable)
                
                class_name = variables[variable].get_value_class()
                variable_node.setAttribute("class", class_name)
                variable_node.setAttribute("value", pickle.dumps(variables[variable].get_value()).encode('base64').replace('\n', ''))
                
                category_node.appendChild(variable_node)
            
            root_node.appendChild(category_node)
        
        dom_tree.appendChild(root_node)
        
        fd = open(self._file_name, "w")
        fd.write(dom_tree.toprettyxml(indent="   "))
        fd.close()
    
    











