
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from collections import deque
from threading import Semaphore, Thread

class OutputManager(object):
    
    def __init__(self, core):
        """Initialize message deques, attributes and plugin list."""
        self._core = core
        self._information_queue = deque()
        self._information_semaphore = Semaphore(0)
        self._debug_queue = deque()
        self._debug_semaphore = Semaphore(0)
        self._error_queue = deque()
        self._error_semaphore = Semaphore(0)
        self._log_queue = deque()
        self._log_semaphore = Semaphore(0)
        self._vulnerability_queue = deque()
        self._vulnerability_semaphore = Semaphore(0)
        self._plugins = None
        self._terminate = False
        self._threads = []
    
    def initialize(self, plugins):
        self._plugins = plugins
        thread = Thread(target=self._consumer_routine, args=(self._information_queue,
                                                             self._information_semaphore,
                                                             'information'))
        self._threads.append(thread)
        thread = Thread(target=self._consumer_routine, args=(self._debug_queue,
                                                             self._debug_semaphore,
                                                             'debug'))
        self._threads.append(thread)
        thread = Thread(target=self._consumer_routine, args=(self._error_queue,
                                                             self._error_semaphore,
                                                             'error'))
        self._threads.append(thread)
        thread = Thread(target=self._consumer_routine, args=(self._log_queue,
                                                             self._log_semaphore,
                                                             'log'))
        self._threads.append(thread)
        thread = Thread(target=self._consumer_routine, args=(self._vulnerability_queue,
                                                             self._vulnerability_semaphore,
                                                             'vulnerability'))
        self._threads.append(thread)
        for thread in self._threads:
            thread.start()
    
    def clean(self):
        self._terminate = True
        self._information_semaphore.release()
        self._debug_semaphore.release()
        self._error_semaphore.release()
        self._log_semaphore.release()
        self._vulnerability_semaphore.release()
        for thread in self._threads:
            thread.join()
        self._terminate = False
        self._threads = []
        self._information_queue = deque()
        self._information_semaphore = Semaphore(0)
        self._debug_queue = deque()
        self._debug_semaphore = Semaphore(0)
        self._error_queue = deque()
        self._error_semaphore = Semaphore(0)
        self._log_queue = deque()
        self._log_semaphore = Semaphore(0)
        self._vulnerability_queue = deque()
        self._vulnerability_semaphore = Semaphore(0)
        for plugin in self._plugins:
            plugin.clean()
    
    def debug(self, message):
        self._debug_queue.append(message)
        self._debug_semaphore.release()
    
    def information(self, message):
        self._information_queue.append(message)
        self._information_semaphore.release()
    
    def error(self, message):
        self._error_queue.append(message)
        self._error_semaphore.release()
    
    def log(self, message):
        self._log_queue.append(message)
        self._log_semaphore.release()
    
    def vulnerability(self, vuln):
        self._vulnerability_queue.append(vuln)
        self._vulnerability_semaphore.release()

    def _consumer_routine(self, deque, semaphore, send_message):
        while True:
            semaphore.acquire()
            if self._terminate:
                break
            message = deque.popleft()
            for plugin in self._plugins:
                getattr(plugin, send_message)(message)
        while deque:
            message = deque.popleft()
            for plugin in self._plugins:
                getattr(plugin, send_message)(message)
    
    