
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri, Carlos Gaston Traberg
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from core.audit_subsystem.AuditPlugin import AuditPlugin
from core.request_subsystem.Request import Request
from core.audit_subsystem.Vulnerability import Vulnerability
import core.constants.dbms as dbms
import core.constants.severities as severity
from core.constants.vulnerabilities import BLIND_SQLI
from core.constants.option_types import INTEGER, FLOAT

class statement(object):
    def __init__(self, sql_command, dbms):
        self.sql_command = sql_command
        self.dbms = dbms
        self.asked = 0
        self.weight = 0

    def get_param_str(self, origParam, value):
        self.asked += 1
        return self.sql_command.format(origParam, value, self.asked)
    
    def inc_weight(self, amount):
        self.weight += amount
    
    def get_weight(cls, statement):
        return -statement.weight

class BlindSQLInjection(AuditPlugin):
    
    options = (
        AuditPlugin.create_option('audit.BlindSQLInjection', 'delay_time', 6, INTEGER),
        AuditPlugin.create_option('audit.BlindSQLInjection', 'tolerance', 0.2, FLOAT),
    )
    
    def __init__(self):
        AuditPlugin.__init__(self)
        self._statements = self.__class__._statements
    
    def audit(self, request):
        #self.get_core().get_output_manager().debug("Starting to audit " + request.get_url())
        
        for parameter in request.get_parameters():
            #self.get_core().get_output_manager().debug("Start to audit " + request.get_url() + " -> " + parameter)
            vuln = self._audit_param(request, parameter)
            if vuln:
                self.get_core().get_output_manager().vulnerability(vuln)
            #self.get_core().get_output_manager().debug("Finishing to audit " + request.get_url() + " -> " + parameter)
        #self.get_core().get_output_manager().debug("Finishing to audit " + request.get_url())
    
    def _audit_param(self, request, parameter):
        
        #Do a basic test to find possible SQL Injections strings
        suspects = []
        #Gather the original value of the variable to use it later.
        original_var_value = request.get_parameters()[parameter]
        response = self._send(request)
        if response is None:
            return None
        base_time = response.get_time()
        suspect = False
        for statement in self._statements:
            new_parameters = request.get_parameters()
            #Sending with a high value
            if statement.dbms == dbms.MYSQL4:
                #In case it's a MySQL4, it should be treated differently
                new_parameters[parameter] = statement.get_param_str(original_var_value, 1000000)
            else:
                #Other cases it should be treated with seconds
                new_parameters[parameter] = statement.get_param_str(original_var_value, 5)
            new_request = Request(request.get_method(),
                                  request.get_url(),
                                  new_parameters,
                                  request.get_headers()
                                  )
            
            response = self._send(new_request)
            high_time = response.get_time()
            if (high_time > base_time + 2):
                suspect = True
                break
        
        if suspect == False:
            return None
        
        #First get the average response time with 1 second delay or doing
        #almost nothing for MySQL
        if statement.dbms == dbms.MYSQL4:
            #If it is a MySQL4 statement it should be treated differently because it doesn't
            #have a delay function.
            
            benchs = 250000
            
            #Get the response time for a small number of benchmarks.
            new_parameters[parameter] = statement.get_param_str(original_var_value, benchs)
            new_request = Request(request.get_method(),
                                  request.get_url(),
                                  new_parameters,
                                  request.get_headers()
                                  )
            response = self._send(new_request)
            response_time = response.get_time()
            
            #Based on response time, we can determine how many iterations we
            #need to force the server to do in order to get _wait_time seconds of delay
            diff = response_time - base_time
            secs_per_benchmark = response_time / benchs
            waiting_time = self.get_option('delay_time').get_value() - base_time
            benchs_todo = waiting_time / secs_per_benchmark
            
            #Get the response time for the calculated number of iterations
            new_parameters[parameter] = statement.get_param_str(original_var_value, benchs_todo)
            
        else:
            #Calculate the response time with a delay of_wait_time 6 seconds
            new_parameters[parameter] = statement.get_param_str(original_var_value, self.get_option('delay_time').get_value())
        
        new_request = Request(request.get_method(),
                              request.get_url(),
                              new_parameters,
                              request.get_headers()
                              )
        
        response = self._send(new_request)
        time_for_test = response.get_time()
        
        #If time_for_test is around (_error * _wait_time) seconds
        #Create the vulnerability object and return it.
        time_for_test -= base_time
        margin = self.get_option('delay_time').get_value() * self.get_option('tolerance').get_value()
        if time_for_test > (self.get_option('delay_time').get_value() - margin) and time_for_test < (self.get_option('delay_time').get_value() + margin):
            #Create the vulnerability object and return it
            vuln = Vulnerability(new_request, BLIND_SQLI, severity.HIGH, parameter)
            #We increase the weight of the statements that belong to the
            #same database as the mathching sqli statement.
            for stmt in [ x for x in self._statements if x.dbms == statement.dbms]:
                stmt.inc_weight(1)

            #We increase one more time the statement that generated the
            #sqli injection, so it has more weight that the others.
            #Then we sort the statement list.
            statement.inc_weight(1)
            self._statements.sort(key=statement.get_weight)
            
            return vuln
        return None

    _statements = [
        statement("{0}   ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0})  ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0})) ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}'  ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}') ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}'));waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}\"  ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}\") ;waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0}\"));waitfor delay '0:0:{1}'-- {2}", dbms.SQL_SERVER),
        statement("{0} and SLEEP({1}) and {2}={2} ", dbms.MYSQL5),
        statement("{0} and SLEEP({1}) and {2}={2} -- ", dbms.MYSQL5),
        statement("{0}' and SLEEP({1}) and '{2}'='{2}", dbms.MYSQL5),
        statement('{0}" and SLEEP({1}) and "{2}"="{2}', dbms.MYSQL5),
        statement('-8827372162 or SLEEP({1}) = 0 and {2}={2} limit 1 -- ', dbms.MYSQL5),
        statement("aldkhalksdha5654al' or SLEEP({1}) = 0 and {2}={2} limit 1 -- ", dbms.MYSQL5),
        statement("{0} and BENCHMARK({1},MD5(1)) and {2}={2} --  ", dbms.MYSQL4),
        statement("{0} and BENCHMARK({1},MD5(1)) and {2}={2} ", dbms.MYSQL4),
        statement("{0}' and BENCHMARK({1},MD5(1)) and '{2}'='{2}", dbms.MYSQL4),
        statement('{0}" and BENCHMARK({1},MD5(1)) and "{2}"="{2}', dbms.MYSQL4),
        statement('-8827372162 or BENCHMARK({1},MD5(1)) = 0 and {2}={2} limit 1 -- ', dbms.MYSQL4),
        statement("aldkhalksdha5654al' or BENCHMARK({1},MD5(1)) = 0 and {2}={2} limit 1 -- ", dbms.MYSQL4),
        statement("{0}  and pg_sleep({1}) and {2}={2} "   , dbms.POSTGRES),
        statement("{0}' and pg_sleep({1}) and '{2}'='{2}" , dbms.POSTGRES),
        statement('{0}" and pg_sleep({1}) and "{2}"="{2}' , dbms.POSTGRES),
        
    ]

