
###############################################################################
#
#    RABBIT Vulnerability Scanner. Black-Box interactive vulnerability scanner.
#    Copyright (C) 2010  Santiago Alessandri(CERT.UNLP)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import datetime
import xml.dom.minidom
from threading import Lock

from core.output_subsystem.OutputPlugin import OutputPlugin
from core.constants.option_types import STRING
from core.constants.severities import HIGH, MEDIUM, LOW

class HTMLOutput(OutputPlugin):
    
    options = (
        OutputPlugin.create_option('output.HTMLOutput', 'filename', 'output.html', STRING),
        OutputPlugin.create_option('output.HTMLOutput', 'debug', 'TRUE', STRING),
        OutputPlugin.create_option('output.HTMLOutput', 'error', 'TRUE', STRING),
        OutputPlugin.create_option('output.HTMLOutput', 'log', 'TRUE', STRING),
        OutputPlugin.create_option('output.HTMLOutput', 'information', 'TRUE', STRING),
        OutputPlugin.create_option('output.HTMLOutput', 'vulnerability', 'TRUE', STRING),
    )
    
    def __init__(self):
        OutputPlugin.__init__(self)
        self._file = open(self.get_option('filename').get_value(), "w")
        self._mutex = Lock()
        self._make_basic_structure()
        

    def debug(self, message):
        if self.get_option('debug').get_value() != 'TRUE':
            return
        with self._mutex:
            row = self._create_element("tr")
            row.appendChild(self._create_element("td", str(datetime.datetime.now())))
            row.appendChild(self._create_element("td", message))
            self._debug_node.appendChild(row)
            self._file.truncate(0)
            self._file.write(self._html_tree.toprettyxml(indent="    "))
            self._file.flush()
    
    def information(self, message):
        if self.get_option('information').get_value() != 'TRUE':
            return
        with self._mutex:
            row = self._create_element("tr")
            row.appendChild(self._create_element("td", str(datetime.datetime.now())))
            row.appendChild(self._create_element("td", message))
            self._information_node.appendChild(row)
            self._file.truncate(0)
            self._file.write(self._html_tree.toprettyxml(indent="    "))
            self._file.flush()
    
    def error(self, message):
        if self.get_option('error').get_value() != 'TRUE':
            return
        with self._mutex:
            row = self._create_element("tr")
            row.appendChild(self._create_element("td", str(datetime.datetime.now())))
            row.appendChild(self._create_element("td", message))
            self._errors_node.appendChild(row)
            self._file.truncate(0)
            self._file.write(self._html_tree.toprettyxml(indent="    "))
            self._file.flush()
    
    def log(self, message):
        if self.get_option('log').get_value() != 'TRUE':
            return
        with self._mutex:
            row = self._create_element("tr")
            row.appendChild(self._create_element("td", str(datetime.datetime.now())))
            row.appendChild(self._create_element("td", message))
            self._log_node.appendChild(row)
            self._file.truncate(0)
            self._file.write(self._html_tree.toprettyxml(indent="    "))
            self._file.flush()
    
    def vulnerability(self, vuln):
        if self.get_option('vulnerability').get_value() != 'TRUE':
            return
        if vuln.get_severity() == HIGH:
            table = self._high_vuln_node
        elif vuln.get_severity() == MEDIUM:
            table = self._med_vuln_node
        else:
            table = self._low_vuln_node
        with self._mutex:
            row = self._create_element("tr")
            row.appendChild(self._create_element("td", str(datetime.datetime.now())))
            row.appendChild(self._create_element("td", str(vuln.get_id())))
            row.appendChild(self._create_element("td", vuln.get_type()))
            row.appendChild(self._create_element("td", vuln.get_request().get_method()))
            row.appendChild(self._create_element("td", vuln.get_request().get_url()))
            row.appendChild(self._create_element("td", vuln.get_parameter()))
            row.appendChild(self._create_element("td", vuln.get_request().get_parameters().get(vuln.get_parameter(), '')))
            table.appendChild(row)
            self._file.truncate(0)
            self._file.write(self._html_tree.toprettyxml(indent="    "))
            self._file.flush()
    
    def clean(self):
        self._file.close()
    
    def _make_basic_structure(self):
        self._html_tree = xml.dom.minidom.Document()
        html_node = self._create_element('html')
        self._html_tree.appendChild(html_node)
        head_node = self._create_element("head")
        title_node = self._create_element("title", "RABBIT Web Vulnerability Scanner - Audit Results")
        head_node.appendChild(title_node)
        html_node.appendChild(head_node)
        body_node = self._create_element("body")
        html_node.appendChild(body_node)
        div_node = self._create_element("div")
        div_node.setAttribute("class", "active_plugins")
        body_node.appendChild(div_node)
        #TODO: Insert each active plugin with its options.
        
        div_node = self._create_element("div")
        div_node.setAttribute("class", "vulnerabilities")
        h3_node = self._create_element("h3", "Vulnerabilities")
        div_node.appendChild(h3_node)
        h4_node = self._create_element("h4", "High Severity Vulnerabilities")
        div_node.appendChild(h4_node)
        high_vuln_node = self._create_element("table")
        high_vuln_node.setAttribute("id", "high_vuln")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Id")
        head_row.appendChild(header)
        header = self._create_element("th", "Type")
        head_row.appendChild(header)
        header = self._create_element("th", "Method")
        head_row.appendChild(header)
        header = self._create_element("th", "URL")
        head_row.appendChild(header)
        header = self._create_element("th", "Parameter")
        head_row.appendChild(header)
        header = self._create_element("th", "Injected Value")
        head_row.appendChild(header)
        high_vuln_node.appendChild(head_row)
        div_node.appendChild(high_vuln_node)
        h4_node = self._create_element("h4", "Medium Severity Vulnerabilities")
        div_node.appendChild(h4_node)
        med_vuln_node = self._create_element("table")
        med_vuln_node.setAttribute("id", "medium_vuln")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Id")
        head_row.appendChild(header)
        header = self._create_element("th", "Type")
        head_row.appendChild(header)
        header = self._create_element("th", "Method")
        head_row.appendChild(header)
        header = self._create_element("th", "URL")
        head_row.appendChild(header)
        header = self._create_element("th", "Parameter")
        head_row.appendChild(header)
        header = self._create_element("th", "Injected Value")
        head_row.appendChild(header)
        med_vuln_node.appendChild(head_row)
        div_node.appendChild(med_vuln_node)
        h4_node = self._create_element("h4", "Low Severity Vulnerabilities")
        div_node.appendChild(h4_node)
        low_vuln_node = self._create_element("table")
        low_vuln_node.setAttribute("id", "low_vuln")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Id")
        head_row.appendChild(header)
        header = self._create_element("th", "Type")
        head_row.appendChild(header)
        header = self._create_element("th", "Method")
        head_row.appendChild(header)
        header = self._create_element("th", "URL")
        head_row.appendChild(header)
        header = self._create_element("th", "Parameter")
        head_row.appendChild(header)
        header = self._create_element("th", "Injected Value")
        head_row.appendChild(header)
        low_vuln_node.appendChild(head_row)
        div_node.appendChild(low_vuln_node)
        body_node.appendChild(div_node)
        self._high_vuln_node = high_vuln_node
        self._med_vuln_node = med_vuln_node
        self._low_vuln_node = low_vuln_node
        
        div_node = self._create_element("div")
        div_node.setAttribute("class", "information")
        h3_node = self._create_element("h3", "Information")
        div_node.appendChild(h3_node)
        table_node = self._create_element("table")
        table_node.setAttribute("id", "info_table")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Message")
        head_row.appendChild(header)
        table_node.appendChild(head_row)
        div_node.appendChild(table_node)
        body_node.appendChild(div_node)
        self._information_node = table_node
        
        div_node = self._create_element("div")
        div_node.setAttribute("class", "errors")
        h3_node = self._create_element("h3", "Errors")
        div_node.appendChild(h3_node)
        table_node = self._create_element("table")
        table_node.setAttribute("id", "error_table")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Message")
        head_row.appendChild(header)
        table_node.appendChild(head_row)
        div_node.appendChild(table_node)
        body_node.appendChild(div_node)
        self._errors_node = table_node
        
        div_node = self._create_element("div")
        div_node.setAttribute("class", "log")
        h3_node = self._create_element("h3", "Log Information")
        div_node.appendChild(h3_node)
        table_node = self._create_element("table")
        table_node.setAttribute("id", "log_table")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Message")
        head_row.appendChild(header)
        table_node.appendChild(head_row)
        div_node.appendChild(table_node)
        body_node.appendChild(div_node)
        self._log_node = table_node
        
        div_node = self._create_element("div")
        div_node.setAttribute("class", "debug")
        h3_node = self._create_element("h3", "Debug Information")
        div_node.appendChild(h3_node)
        table_node = self._create_element("table")
        table_node.setAttribute("id", "debug_table")
        head_row = self._create_element("tr")
        header = self._create_element("th", "Time")
        head_row.appendChild(header)
        header = self._create_element("th", "Message")
        head_row.appendChild(header)
        table_node.appendChild(head_row)
        div_node.appendChild(table_node)
        body_node.appendChild(div_node)
        self._debug_node = table_node
    
    def _create_element(self, tag, text=''):
        tag_node = self._html_tree.createElement(tag)
        if text:
            text_node = self._html_tree.createTextNode(text)
            tag_node.appendChild(text_node)
        return tag_node
